<?php

use Nette\Forms\Form;
use Instante\Bootstrap3Renderer\BootstrapRenderer;
use Nette\Security\Passwords;

function find_user($id) {
    global $app, $db;

    require_user_access($id);
    $dbuser = $db->table('user')->get($id);

    if (!$dbuser)
        $app->halt(403, _('User not found'));

    return $dbuser;
}

function edit_user_form($dbuser) {
    $form = new Form;
    $form->setRenderer(new BootstrapRenderer);
    $form->setAction("user/$dbuser->id");
    $form->addHidden('_METHOD', 'PUT');

    $check_current_password = function ($password) use ($dbuser) {
        return Passwords::verify($password->getValue(), $dbuser->password);
    };

    $form->addPassword('current_password', _('Current password'))
        ->setRequired()
        ->addRule($check_current_password, _('Password incorrect'));
    $form->addPassword('new_password', _('New password'))
        ->addCondition(Form::FILLED)
            ->addRule(Form::MIN_LENGTH, sprintf(_('Password must be at least %d charachers'),
                MIN_PASSWORD_CHARS), MIN_PASSWORD_CHARS);
    $form->addPassword('new_password_repeat', _('Confirm new password'))
        ->addConditionOn($form['new_password'], Form::FILLED)
            ->addRule(Form::EQUAL, _('Passwords must match'), $form['new_password']);
    $form->addSubmit('send', _('Save'))->setAttribute('class', 'btn-primary');

    return $form;
}

$app->get('/user/:id', function ($id) use ($app, $db) {
    $app->render('user/view', array('dbuser' => find_user($id)));
});

$app->get('/user/:id/edit', function ($id) use ($app) {
    $dbuser = find_user($id);
    $form = edit_user_form($dbuser);
    $app->render('user/edit', compact('dbuser', 'form'));
});
$app->put('/user/:id', function ($id) use ($app, $user) {
    $dbuser = find_user($id);
    $form = edit_user_form($dbuser);
    $form->validate();

    if (!$form->hasErrors()) {
        $values = $form->getValues();

        if ($values->new_password) {
            $dbuser->update(array(
                'password' => Passwords::hash($values->new_password)
            ));
            //$user->logout();
            //$user->login($dbuser->username, $values->new_password);
        }

        $app->redirect(ROOT_URL . "/user/$id");
    }

    $app->render('user/edit', compact('dbuser', 'form'));
});

$app->put('/user/:id', function ($id) {
    echo "update user $id";
});

$app->delete('/user/:id', function ($id) {
    echo "delete user $id";
});

function plot_filter_form($dbuser, $matches, $default_discipline) {
    global $db;

    $disciplines = array('barebow', 'recurve', 'compound');
    $disciplines = array_combine($disciplines, array_map('ucfirst', $disciplines));

    $match_ids = array_unique($matches->fetchPairs(null, 'id'));
    $tags = $db->table('tag')
        ->select('DISTINCT name AS name')
        ->where(array('match_id' => $match_ids))
        ->fetchPairs('name', 'name');
    asort($tags);

    $distances = $matches->fetchPairs('distance', 'distance');
    asort($distances);
    $distances = map_format(_('%d meters'), $distances);

    $form = new Form;
    $form->setRenderer(new BootstrapRenderer);
    $form->setAction("user/$dbuser->id/plot");
    $form->setMethod('get');

    $form->addText('from', _('From'))
        ->setType('date')
        ->setAttribute('placeholder', _('YYYY-MM-DD'));
    $form->addText('until', _('Until'))
        ->setType('date')
        ->setAttribute('placeholder', _('YYYY-MM-DD'))
        ->setDefaultValue(strftime('%Y-%m-%d'));
    $form->addMultiSelect('tags', _('Tags'), $tags);
    $form->addSelect('distance', _('Distance'), $distances)
        ->setPrompt(_('Select a distance'));
    $form->addRadioList('discipline', _('Discipline'), $disciplines)
        ->setAttribute('data-inline', true)
        ->setDefaultValue($default_discipline);
    $form->addSubmit('send', _('Filter'));

    return $form;
}

$app->get('/user/:id/plot', function ($id) use ($app, $db, $user) {
    $dbuser = find_user($id);

    $matches = $db->table('match')
        ->where(array('user_id' => $id))
        ->order('created_at DESC');

    if ($matches->count()) {
        $last_match = $matches->fetch();
        $default_discipline = $last_match->discipline;
        $matches->rewind();
    } else {
        $default_discipline = 'recurve';
    }

    $form = plot_filter_form($dbuser, $matches, $default_discipline);

    if ($form->isSubmitted()) {
        $v = $form->getValues();

        if ($v->tags) {
            $match_ids = $db->table('tag')
                ->select('match_id')
                ->where('name', $v->tags)
                ->fetchPairs(null, 'match_id');
            $matches->where('id', $match_ids);
        }

        if ($v->from && $v->until)
            $matches->where('DATE(created_at) BETWEEN ? AND ?', $v->from, $v->until);
        elseif ($v->from)
            $matches->where('DATE(created_at) >= ?', $v->from);
        elseif ($v->until)
            $matches->where('DATE(created_at) <= ?', $v->until);

        if ($v->distance)
            $matches->where('distance', $v->distance);

        if ($v->discipline)
            $matches->where('discipline', $v->discipline);
    } else {
        $matches->where('discipline', $default_discipline);
    }

    $app->render('user/plot', compact('dbuser', 'form', 'matches'));
});
